#include <SDL/SDL.h>
#include <vector>

#include "cMazeCell.h"

#ifndef CMAZE_H
#define CMAZE_H

const int MAZE_MGNR = 1;
const int MAZE_PATH = 2;
const int MAZE_DONE = 3;

class cMaze {
public:
                        // tempo
    virtual int logic(int, bool);
    virtual int generateStep()=0;
    virtual int render(bool);
    virtual int pathfinding();

    virtual ~cMaze(){};

    int dAddFront(int x, int y);

protected:
    std::vector< std::vector<int> > dFronts;
    std::vector< std::vector<int> > dPath;
    cMazeField* maze;
    SDL_Surface* surfEnlarged;

    int lastFrame;
    int mazeState;
    int pathState;

    int dCurX, dCurY;
} ;

const int PATH_FIND = 0;
const int PATH_BACK = 1;
const int PATH_FINISH = 2;

class cMG : public cMaze {
public:
    int generateStep();
    bool evaluate(int x1, int y1, int x2, int y2);
    bool evaluateW(int x1, int y1, int x2, int y2);
    int addEdge(int x1, int y1, int x2, int y2);
    int deleteEdge(int x1, int y1, int x2, int y2);

    std::vector< std::vector<int> > choices;

    cMG(int size); // set mazestate set lastframe;
    ~cMG(); // Clear vector & mazefield

private:
    int mSize;
} ;

class cBinary : public cMaze {
public:
    int generateStep();

    cBinary(int size); // set mazestate set lastframe;
    ~cBinary(); // Clear vector & mazefield

private:
    int nextX, nextY;
} ;

class cPrims : public cMaze {
public:
    int generateStep();

    cPrims(int size); // set mazestate set lastframe; SET SIZE
    ~cPrims(); // Clear vector & mazefield

private:
    std::vector< std::vector<int> > fronts;

} ;

class cHuntKill : public cMaze {
public:
    int generateStep();

    cHuntKill(int size);
    ~cHuntKill();

private:
    int curX;
    int curY;
    int curRow;
    int curState;
    int minRow;
} ;

const int HK_KILL = 1;
const int HK_HUNT = 2;

#endif
